# -*- coding: utf-8 -*-
#!/usr/bin/env python
from xml.dom import minidom

#========================================
# This code was extracted from Python-Epub 0.5.2
# http://epub.exirel.me/epub/opf.html
# Florian Strzelecki <florian.strzelecki@gmail.com>'
#========================================

XMLNS_DC = 'http://purl.org/dc/elements/1.1/'
XMLNS_OPF = 'http://www.idpf.org/2007/opf'
XMLNS_CALIBRE = "http://calibre.kovidgoyal.net/2009/metadata"
XMLNS_DCTERMS = "http://purl.org/dc/terms/" 
XMLNS_XSI = "http://www.w3.org/2001/XMLSchema-instance"


def get_node_text(node):
    """
    Return the text content of an xml.dom Element Node.

    If node does not have content, this function return an empty string.
    """
    text = ''

    node.normalize()
    if node.firstChild and node.firstChild.data:
        text = node.firstChild.data.strip()

    return text

# parses the metadata section
def _parse_xml_metadata(element):
    """Extract metadata from an xml.dom.Element object (ELEMENT_NODE)

    The "<metadata>" tag has a lot of metadatas about the epub this method
    inspect and store into object attributes (like "title" or "creator").
    """
    metadata = Metadata()

    for node in element.getElementsByTagName('dc:creator'):
        metadata.add_creator(get_node_text(node),
                             node.getAttribute('opf:role'),
                             node.getAttribute('opf:file-as'),
                             node.getAttribute('id'),
                             node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:contributor'):
        metadata.add_contributor(get_node_text(node),
                             node.getAttribute('opf:role'),
                             node.getAttribute('opf:file-as'),
                             node.getAttribute('id'),
                             node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:coverage'):
        metadata.add_coverage(get_node_text(node),
                             node.getAttribute('id'),                           
                             node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:date'):
        metadata.add_date(get_node_text(node),
                          node.getAttribute('opf:event'),
                          node.getAttribute('id'))

    for node in element.getElementsByTagName('dc:description'):
        metadata.add_description(get_node_text(node),
                          node.getAttribute('id'),                           
                          node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:format'):
        metadata.add_format(get_node_text(node),
                          node.getAttribute('id'))

    for node in element.getElementsByTagName('dc:identifier'):
        metadata.add_identifier(get_node_text(node),
                            node.getAttribute('id'),
                            node.getAttribute('opf:scheme'))

    for node in element.getElementsByTagName('dc:language'):
        metadata.add_language(get_node_text(node),
                          node.getAttribute('id'))

    for node in element.getElementsByTagName('dc:publisher'):
        metadata.add_publisher(get_node_text(node),
                          node.getAttribute('id'),                           
                          node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:relation'):
        metadata.add_relation(get_node_text(node),
                          node.getAttribute('id'),                           
                          node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:rights'):
        metadata.add_right(get_node_text(node),
                          node.getAttribute('id'),                           
                          node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:source'):
        metadata.add_source(get_node_text(node),
                          node.getAttribute('id'),                           
                          node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:subject'):
        metadata.add_subject(get_node_text(node),
                          node.getAttribute('id'),                           
                          node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:title'):
        metadata.add_title(get_node_text(node),
                             node.getAttribute('id'),                           
                             node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:type'):
        metadata.add_dc_type(get_node_text(node),
                             node.getAttribute('id'))   

    for node in element.getElementsByTagName('meta'):
        metadata.add_meta(node.getAttribute('name'),
                          node.getAttribute('content'),
                          node.getAttribute('opf:scheme'),                          
                          node.getAttribute('id'),                           
                          node.getAttribute('xml:lang'))

    return metadata

# the main class
class Metadata(object):
    """Represent an epub's metadatas set.
    See http://www.idpf.org/epub/20/spec/OPF_2.0_latest.htm#Section2.2"""

    def __init__(self):
        self.creators = []
        self.contributors = []
        self.coverage = []
        self.dates = []
        self.description = []
        self.dc_format = []
        self.identifiers = []
        self.languages = []
        self.publisher = []
        self.relation = []
        self.right = []
        self.source = []
        self.subjects = []
        self.titles = []
        self.dc_type = []
        self.metas = []

    def add_creator(self, name, role=None, file_as=None, id=None, lang=None):
        role = role or ''
        file_as = file_as or ''
        id = id or ''
        lang = lang or ''        
        self.creators.append((name, role, file_as, id, lang))

    def add_contributor(self, name, role=None, file_as=None, id=None, lang=None):
        role = role or ''
        file_as = file_as or ''
        id = id or ''
        lang = lang or ''           
        self.contributors.append((name, role, file_as, id, lang))

    def add_coverage(self, coverage, id=None, lang=None):
        id = id or ''
        lang = lang or ''        
        self.coverage.append((coverage, id, lang))

    def add_date(self, date, event=None, id=None):
        event = event or ''
        id = id or ''
        self.dates.append((date, event, id))

    def add_description(self, description, id=None, lang=None):
        id = id or ''
        lang = lang or ''        
        self.description.append((description, id, lang))

    def add_format(self, format, id=None):
        id = id or ''
        self.dc_format.append((format, id))
        
    def add_identifier(self, content, id=None, scheme=None):
        id = id or ''
        scheme = scheme or ''
        self.identifiers.append((content, id, scheme))

    def add_language(self, lang, id=None):
        id = id or '' 
        self.languages.append((lang, id))

    def add_publisher(self, publisher, id=None, lang=None):
        id = id or ''
        lang = lang or ''        
        self.publisher.append((publisher, id, lang))

    def add_relation(self, relation, id=None, lang=None):
        id = id or ''
        lang = lang or ''        
        self.relation.append((relation, id, lang))

    def add_right(self, right, id=None, lang=None):
        id = id or ''
        lang = lang or ''        
        self.right.append((right, id, lang))

    def add_source(self, source, id=None, lang=None):
        id = id or ''
        lang = lang or ''        
        self.source.append((source, id, lang))

    def add_subject(self, subject, id=None, lang=None):
        id = id or ''
        lang = lang or ''        
        self.subjects.append((subject, id, lang))

    def add_title(self, title, id=None, lang=None):
        id = id or ''
        lang = lang or ''        
        self.titles.append((title, id, lang))

    def add_dc_type(self, dc_type, id=None):
        id = id or ''
        self.dc_type.append((dc_type, id))

    def add_meta(self, name, content, scheme=None, id=None, lang=None):
        scheme = scheme or ''
        id = id or ''
        lang = lang or ''        
        self.metas.append((name, content, scheme, id, lang))

    def get_isbn(self):
        l = [x[0] for x in self.identifiers if x[2].lower() == 'isbn']
        isbn = None
        if l:
            isbn = l[0]
        return isbn

    def as_xml_element(self):
        """Return an xml dom Element node."""
        doc = minidom.Document()
        metadata = doc.createElement('metadata')
        metadata.setAttribute('xmlns:dc', XMLNS_DC)
        metadata.setAttribute('xmlns:opf', XMLNS_OPF)

        for text, id, lang in self.titles:
            title = doc.createElement('dc:title')
            if id:
                title.setAttribute('id', id)
            if lang:
                title.setAttribute('xml:lang', lang)
            title.appendChild(doc.createTextNode(text))
            metadata.appendChild(title)
        
        for name, role, file_as, id, lang in self.creators:
            creator = doc.createElement('dc:creator')
            if role:
                creator.setAttribute('opf:role', role)
            if file_as:
                creator.setAttribute('opf:file-as', file_as)
            if id:
                creator.setAttribute('id', id)
            if lang:
                creator.setAttribute('xml:lang', lang)
            creator.appendChild(doc.createTextNode(name))
            metadata.appendChild(creator)

        for name, role, file_as, id, lang in self.contributors:
            if name.startswith('calibre'):
                metadata.setAttribute('xmlns:calibre', XMLNS_CALIBRE)
            contributor = doc.createElement('dc:contributor')
            if role:
                contributor.setAttribute('opf:role', role)
            if file_as:
                contributor.setAttribute('opf:file-as', file_as)
            if id:
                contributor.setAttribute('id', id)
            if lang:
                contributor.setAttribute('xml:lang', lang)
            contributor.appendChild(doc.createTextNode(name))
            metadata.appendChild(contributor)

        for text, id in self.languages:
            language = doc.createElement('dc:language')
            if id:
                language.setAttribute('id', id)
            language.appendChild(doc.createTextNode(text))
            metadata.appendChild(language)

        for text, id, lang in self.publisher:
            publisher = doc.createElement('dc:publisher')
            if id:
                publisher.setAttribute('id', id)
            if lang:
                publisher.setAttribute('xml:lang', lang)
            publisher.appendChild(doc.createTextNode(text))
            metadata.appendChild(publisher)

        for text, id, scheme in self.identifiers:
            dc_identifier = doc.createElement('dc:identifier')
            if id:
                dc_identifier.setAttribute('id', id)
            if scheme:
                dc_identifier.setAttribute('opf:scheme', scheme)
            dc_identifier.appendChild(doc.createTextNode(text))
            metadata.appendChild(dc_identifier)
            
        for text, event, id in self.dates:
            date = doc.createElement('dc:date')
            if event:
                date.setAttribute('opf:event', event)
            if id:
                date.setAttribute('id', id)
            date.appendChild(doc.createTextNode(text))
            metadata.appendChild(date)

        for text, id, lang in self.description:
            description = doc.createElement('dc:description')
            if id:
                description.setAttribute('id', id)
            if lang:
                description.setAttribute('xml:lang', lang)
            description.appendChild(doc.createTextNode(text))
            metadata.appendChild(description)

        for text, id, lang in self.subjects:
            subject = doc.createElement('dc:subject')
            if id:
                subject.setAttribute('id', id)
            if lang:
                subject.setAttribute('xml:lang', lang)
            subject.appendChild(doc.createTextNode(text))
            metadata.appendChild(subject)

        for text, id, lang in self.right:
            right = doc.createElement('dc:rights')
            if id:
                right.setAttribute('id', id)
            if lang:
                right.setAttribute('xml:lang', lang)
            right.appendChild(doc.createTextNode(text))
            metadata.appendChild(right)
            
        for text, id, lang in self.coverage:
            coverage = doc.createElement('dc:coverage')
            if id:
                coverage.setAttribute('id', id)
            if lang:
                coverage.setAttribute('xml:lang', lang)
            coverage.appendChild(doc.createTextNode(text))
            metadata.appendChild(coverage)


        for text, id in self.dc_format:
            format = doc.createElement('dc:format')
            if id:
                format.setAttribute('id', id)
            format.appendChild(doc.createTextNode(text))
            metadata.appendChild(format)

        for text, id, lang in self.relation:
            relation = doc.createElement('dc:relation')
            if id:
                relation.setAttribute('id', id)
            if lang:
                relation.setAttribute('xml:lang', lang)
            relation.appendChild(doc.createTextNode(text))
            metadata.appendChild(relation)


        for text, id, lang in self.source:
            source = doc.createElement('dc:source')
            if id:
                source.setAttribute('id', id)
            if lang:
                source.setAttribute('xml:lang', lang)
            source.appendChild(doc.createTextNode(text))
            metadata.appendChild(source)

        for text, id in self.dc_type:
            dc_type = doc.createElement('dc:type')
            if id:
                dc_type.setAttribute('id', id)
            dc_type.appendChild(doc.createTextNode(text))
            metadata.appendChild(dc_type)

        for name, content, scheme, id, lang in self.metas:
            if name.startswith('calibre'):
                metadata.setAttribute('xmlns:calibre', XMLNS_CALIBRE)
            meta = doc.createElement('meta')
            meta.setAttribute('name', name)
            meta.setAttribute('content', content)
            if scheme:
                meta.setAttribute('opf:scheme', scheme)
            if id:
                meta.setAttribute('id', id)
            if lang:
                meta.setAttribute('xml:lang', lang)
            metadata.appendChild(meta)

        return metadata
